import subprocess

import pytest
from llm.plugins import pm

from llm_gemini_youtube import is_youtube_uri


@pytest.mark.skip("Need to fix this failing test")
def test_plugin_is_installed():
    names = [mod.__name__ for mod in pm.get_plugins()]
    assert "llm_gemini_youtube" in names


class TestIsYouTubeUri:
    @pytest.mark.parametrize(
        "uri",
        [
            "https://www.youtube.com/watch?v=9hE5-98ZeCg",
            "https://youtu.be/9hE5-98ZeCg",
            "https://www.youtube.com/shorts/46ycw2pQJCA",
        ],
    )
    def test_youtube_uri(self, uri):
        assert is_youtube_uri(uri)

    def test_not_youtube_uri(self):
        assert not is_youtube_uri("https://example.com")


class TestSupportedModels:
    @pytest.mark.parametrize(
        "expected_model",
        [
            "gemini-2.0-flash-yt",
            "gemini-1.5-pro-yt",
            "gemini-2.5-pro-exp-03-25-yt",
        ],
    )
    def test_contains_llm_models_output(self, expected_model):
        result = subprocess.run(
            ["llm", "models", "-q", "-yt"],
            check=True,
            capture_output=True,
            text=True,
        )

        assert expected_model in result.stdout



import os

import llm
from google.genai import Client, types


@llm.hookimpl
def register_models(register):
    register(GeminiYouTube("gemini-2.0-flash-yt"))
    register(GeminiYouTube("gemini-1.5-pro-yt"))
    register(GeminiYouTube("gemini-2.5-pro-exp-03-25-yt"))


def is_youtube_uri(url: str) -> bool:
    return (
        "youtube.com/watch?v=" in url
        or "youtu.be/" in url
        or "youtube.com/shorts/" in url
    )


class GeminiYouTube(llm.KeyModel):
    needs_key = "gemini"
    key_env_var = "LLM_GEMINI_KEY"
    can_stream = True
    attachment_types = set(["text/html; charset=utf-8"])  # for YouTube URLs

    def __init__(self, model_id: str):
        self.model_id = model_id

    def execute(self, prompt, stream, response, conversation, key):
        if not prompt.attachments:
            raise llm.ModelError("Attachment (YouTube URL) is required.")

        client = Client(api_key=os.getenv("LLM_GEMINI_KEY"))

        youtube_uri = None
        for attachment in prompt.attachments:
            if is_youtube_uri(attachment.url):
                youtube_uri = attachment.url
                break
        if not youtube_uri:
            raise llm.ModelError("YouTube URL attachment is required.")

        streaming = client.models.generate_content_stream(
            model=f"models/{self.model_id.removesuffix('-yt')}",
            contents=types.Content(
                parts=[
                    types.Part(text=prompt.prompt),
                    types.Part(file_data=types.FileData(file_uri=youtube_uri)),
                ]
            ),
        )
        for chunk in streaming:
            yield chunk.text
